@@ -59,9 +59,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#ifdef XFCE_SUPPORT
 #include <libxfce4util/util.h>
-#endif
 
 #include "glade_support.h"
 
@@ -77,6 +75,7 @@
 #include "misc.h"
 #include "refresh.h"
 #include "widgets.h"
+#include "input.h"
 
 /*#define DEBUG*/
 
@@ -129,6 +128,7 @@ void set_book_combo(GtkTreeView * treeview)
     for (tmp=new_list;tmp;tmp=tmp->next){
 	    gchar *g=(gchar *)tmp->data;
 	    g_free(g);
+	    g=NULL;
     }
     g_list_free(new_list);
 }
@@ -153,7 +153,7 @@ void reload_book(GtkTreeView *treeview){
 }
 
 
-char *get_bookfile_path(){
+char *get_bookfile_path(void){
     static char bookmarks_path[_POSIX_PATH_MAX];
     gchar *g;
     xfce_get_userfile_r(bookmarks_path, _POSIX_PATH_MAX-1, "xffm");
@@ -163,6 +163,7 @@ char *get_bookfile_path(){
 	    xfce_get_userfile_r(bookmarks_path, _POSIX_PATH_MAX-1,
 		    g);
 	    g_free(g);
+	    g=NULL;
     } else xfce_get_userfile_r(bookmarks_path, _POSIX_PATH_MAX-1,
 		    "xffm%cbookmarks.dbh", G_DIR_SEPARATOR);
 #ifdef DEBUG
@@ -331,11 +332,13 @@ int add2bookmarks(GtkTreeView * treeview, char *path)
 	DBH_close(bookmarks);
 	unset_load_wait(&tree_details);
 	g_free(d_path);
+	d_path=NULL;
 	return TRUE;
     }
     print_diagnostics(treeview, "xf_WARNING_ICON", path, " ", _("already in book"), "\n", NULL);
     DBH_close(bookmarks);
     g_free(d_path);
+    d_path=NULL;
     return FALSE;
 }
 
@@ -456,8 +459,8 @@ int open_bookmarks(GtkTreeView * treeview, GtkTreeIter * iter, GtkTreePath * tre
 	if (!bm_gdir.pathc){
 	   char *bp=get_bookfile_path();
 	   reset_dummy(treeview, iter, 3);
-	   /* do an automatic purge: */
-	   unlink(bp);
+	   /* do an automatic purge:  NO!!!*/
+	   /*unlink(bp);*/
 	}
 	
     }
@@ -600,3 +603,12 @@ void on_remove_from_bookmarks_activate(GtkMenuItem * menuitem, gpointer user_dat
 }
 
 
+void
+on_save_book                           (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+    GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
+    show_input(treeview, SAVE_BOOK_INPUT);
+
+}
+
