@@ -71,6 +71,7 @@ const gchar *get_selected_chdir(GtkTreeView * treeview)
 {
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     gchar *Cdir=NULL;
+    static char ret_dir[_POSIX_PATH_MAX];
     tree_entry_t *en;
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     
@@ -94,7 +95,10 @@ const gchar *get_selected_chdir(GtkTreeView * treeview)
 	   gchar *g;
 	   if (IS_DIR(en->type)) g=g_strdup(en->path);
 	   else g=g_path_get_dirname(en->path);
-	   if (strcmp(g,".")==0) g_free(g);
+	   if (strcmp(g,".")==0) {
+		   g_free(g);
+		   g=NULL;
+	   }
 	   else Cdir=g;
 	  } 	   
 	}
@@ -108,7 +112,9 @@ const gchar *get_selected_chdir(GtkTreeView * treeview)
 #ifdef DEBUG
     printf("DBG: doing chdir to %s\n",Cdir);
 #endif
-    return Cdir;
+    snprintf(ret_dir,_POSIX_PATH_MAX,"%s",Cdir);
+    g_free(Cdir);
+    return ret_dir;
 }
 
 /* this function always returns a local path (or NULL)*/
@@ -244,7 +250,9 @@ tree_entry_t *stat_entry(char *path, int type)
     {
 	if(lstat(path, st) < 0)
 	{
+	    destroy_entry(en);	    
 	    g_free(st);
+	    st=NULL;
 	    return NULL;
 	}
 	SET_BROKEN_LNK(en->type);
@@ -252,7 +260,13 @@ tree_entry_t *stat_entry(char *path, int type)
 	return en;
     }
     en->st = st;
-    lstat(path, &s);
+    if (lstat(path, &s) < 0){
+	    destroy_entry(en);	    
+	    g_free(st);
+	    st=NULL;
+	    return NULL;
+    }
+    
     if(S_ISLNK(s.st_mode))
     {
 	SET_XF_LNK(en->type);
@@ -331,6 +345,7 @@ tree_entry_t *mk_net_entry(char *path,unsigned type){
    en->path=(char *)malloc(strlen(t)+3);
    sprintf(en->path,"//%s",t);
    g_free(p);
+   p=NULL;
    return en;
 }
 
@@ -339,15 +354,24 @@ void destroy_entry(tree_entry_t * en)
 {
     if(!en)
 	return;
-    if(en->filter)
+    if(en->filter){
 	g_free(en->filter);
-    if(en->st)
+	en->filter=NULL;
+    }
+    if(en->st){
 	g_free(en->st);
-    if(en->path)
+	en->st=NULL;
+    }
+    if(en->path){
 	g_free(en->path);
-    if(en->tag)
+	en->path=NULL;
+    }
+    if(en->tag){
 	g_free(en->tag);
+	en->tag=NULL;
+    }
     g_free(en);
+    en=NULL;
 }
 
 gboolean destroy_node_entry(GtkTreeModel * treemodel, GtkTreePath * path, GtkTreeIter * iter, gpointer user_data)
