@@ -456,7 +456,7 @@ static char *resolve_icon_id(tree_entry_t * en)
 	return ("xf_SOCKET_ICON");
 
     if (IS_NOACCESS(en->type)){
-	    return ("xf_no-access");
+	    return ("xf_NOACCESS_ICON");
     }
 
 
@@ -602,6 +602,7 @@ static GdkPixbuf *resolve_icon(GtkTreeView * treeview,
 		stock_id = g_hash_table_lookup(icon_hash, a);
 	    }
 	    g_free(aloc);
+	    aloc=NULL;
 	}
 	if(!stock_id && strchr(loc, '~'))
 	{
@@ -612,6 +613,7 @@ static GdkPixbuf *resolve_icon(GtkTreeView * treeview,
 		stock_id = g_hash_table_lookup(icon_hash, a);
 	    }
 	    g_free(aloc);
+	    aloc=NULL;
 	}
 	
 	if(stock_id)
@@ -736,7 +738,7 @@ void set_icon(GtkTreeView * treeview, GtkTreeIter * iterator)
     if(IS_DUMMY_TYPE(en->type))
 	return;
     /*if (en->path) printf("DBG: icon setting for %s\n",en->path); */
-    /*set_font(treeview,iterator); */
+    set_font(treeview,iterator); 
     Icon = resolve_icon(treeview, en, p_en);
     gtk_tree_store_set((GtkTreeStore *) treemodel, iterator, PIXBUF_COLUMN, Icon, -1);
 }
@@ -820,6 +822,7 @@ void create_icons(tree_details_t * tree_details)
 	printf("      Or select \"plain\" theme to remove this warning.\n");
 	fflush(NULL);
 	g_free(mimefile);
+	mimefile=NULL;
     	mimefile=g_strconcat(PACKAGE_DATA_DIR,"/",PACKAGE,
 			    "/pixmaps/plain/mime.xml",NULL);	    
     } 
@@ -834,6 +837,7 @@ void create_icons(tree_details_t * tree_details)
     theme_dir=g_strconcat(icon_dir,"/",tree_details->theme,NULL);
     if (access(theme_dir,X_OK)!=0){
 	    g_free(theme_dir);
+	    theme_dir=NULL;
     	    theme_dir=g_strconcat(PACKAGE_DATA_DIR,"/",PACKAGE,
 			    "/pixmaps/plain",NULL);	    
     }
@@ -854,17 +858,21 @@ void create_icons(tree_details_t * tree_details)
             printf("DBG: adding pixmapdir=%s\n",subdir);
 #endif
 	    g_free(subdir);
+	    subdir=NULL;
 	    g_free(*t);
+	    *t=NULL;
 	}
 	g_free(themes);
+	themes=NULL;
     }
     add_pixmap_directory(theme_dir);
     g_free(theme_dir);
+    theme_dir=NULL;
 #ifdef DEBUG
             printf("DBG: adding pixmapdir=%s\n",icon_dir);
 #endif
     g_free(icon_dir);
-
+    icon_dir=NULL;
 
     xmlKeepBlanksDefault(0);
 
@@ -900,7 +908,9 @@ void create_icons(tree_details_t * tree_details)
 	    fprintf(stderr,"DBG:adding type: %s %s\n",value,id);
 #endif
 	    g_free(value);
+	    value=NULL;
 	    g_free(id);
+	    id=NULL;
 	}
     }
 
@@ -946,7 +956,9 @@ void create_icons(tree_details_t * tree_details)
 	    gtk_icon_factory_add(icon_factory, id, icon_set);
 
 	    g_free(value);
+	    value=NULL;
 	    g_free(id);
+	    id=NULL;
 	}
 	if(xmlStrEqual(node->name, (const xmlChar *)"search")){
 	    id = xmlGetProp(node, (const xmlChar *)"path");
@@ -956,6 +968,7 @@ void create_icons(tree_details_t * tree_details)
 	            /*printf("DBG: adding %s\n",id);*/
 	    }
 	    g_free(id);
+	    id=NULL;
 	}
     }
 
@@ -963,7 +976,9 @@ void create_icons(tree_details_t * tree_details)
     
 time2return:
     g_free(mimefile);
+    mimefile=NULL;
     g_free(typesfile);
+    typesfile=NULL;
     return;
 
 }
@@ -1027,7 +1042,9 @@ void set_mainmenu_icons(tree_details_t * tree_details)
 static void clear_iconset(gpointer key, gpointer value, gpointer user_data)
 {
     g_free(key);
+    key=NULL;
     g_free(value);
+    value=NULL;
 }
 
 void recreate_icons(tree_details_t * tree_details)
