@@ -52,9 +52,7 @@
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
 
-#ifdef XFCE_SUPPORT
 #include <libxfce4util/util.h>
-#endif
 
 #include "glade_gui.h"
 #include "glade_support.h"
@@ -83,21 +81,19 @@
 
 gchar *workdir=NULL;
 extern gchar *bookfile;
-static GList *list;
-extern gchar *active_dbh_file;
 
-#define CURRENT_WORKDIR_HISTORY "xffm%cxffm.workdir.2.dbh"
 static void save_workdir_history(char *p){
     char fname[_POSIX_PATH_MAX];
     xfce_get_userfile_r(fname, _POSIX_PATH_MAX-1, CURRENT_WORKDIR_HISTORY,
 		    G_DIR_SEPARATOR);
     	save_to_history(fname,p);
 }
-static GList *set_workdir_combo(GtkTreeView * treeview)
+
+static GList *set_workdir_combo(history_combo_info_t *combo_info)
 {
     static GList *g=NULL;
-    char fname[_POSIX_PATH_MAX];
-    tree_details_t *tree_details = get_tree_details(treeview);
+    static char fname[_POSIX_PATH_MAX];
+    tree_details_t *tree_details = get_tree_details(combo_info->treeview);
     GtkWidget *w;
  
     xfce_get_userfile_r(fname, _POSIX_PATH_MAX-1, CURRENT_WORKDIR_HISTORY,
@@ -111,187 +107,22 @@ static GList *set_workdir_combo(GtkTreeView * treeview)
     	save_to_history(fname,(char *)cname);
     }
     get_history_list(&g,fname,"");
-    w = lookup_widget(tree_details->window, "input_combo");
-   
-    gtk_combo_set_popdown_strings(GTK_COMBO(w),g );
+    combo_info->active_dbh_file=fname;
+    combo_info->list=g;
+
+    set_limited_combo(combo_info,NULL);
     return g;
 }
 
-
 static gint on_key_press(GtkWidget * entry, GdkEventKey * event, gpointer data)
 {
-    void ok_input(GtkTreeView * treeview);
-    GtkTreeView *treeview = (GtkTreeView *) data;
-
-    /*printf("DBG(1):got key= 0x%x\n",event->keyval);*/
-    if(event->keyval == GDK_Escape)
-    {
+ GtkTreeView *treeview = (GtkTreeView *)data;
+ /*printf("DBG(2):got key= 0x%x\n",event->keyval);*/
+ if (event->keyval == GDK_Escape) {
 	cancel_input(treeview);
-	print_status(treeview,"xf_INFO_ICON",_("Cancelled"),NULL);
-	gtk_widget_grab_focus((GtkWidget *) treeview);
-	
-	return (TRUE);
-    }
-    return (FALSE);
-}
-
-
-/*summary:
- * control-tab moves to next slash or end of string
- * tab cycles to the next history element that matches
- * control-del removes the item from the history
- * */
-
-static gint on_key_press2(GtkWidget * entry, GdkEventKey * event, gpointer data)
-{
-    void ok_input(GtkTreeView * treeview);
-    gchar *text[2];
-    gchar c[]={0,0};
-    gchar *fulltext=NULL;
-    GList *tmp=list;
-    GtkTreeView *treeview = (GtkTreeView *) data;
-    GtkEditable *editable=(GtkEditable *)entry;
-    gint pos2,pos;
-    gboolean preselection=
-	    gtk_editable_get_selection_bounds (editable,&pos,&pos2);
-    
-    
-    if (!preselection){
-	 pos=gtk_editable_get_position(editable);
-    }	 
-
-    /*printf("DBG(2):pos= %d\n",pos);*/
-   /* printf("DBG(2):got key= 0x%x\n",event->keyval);*/
-
-    /* control tab will move to next slash, 
-     * normal tab will complete on basis
-     * of next candidate in list */
-    
- if (event->keyval == GDK_BackSpace && (event->state&GDK_CONTROL_MASK)){
-	gtk_editable_delete_text (editable,0,-1);
 	return TRUE;
  }
-
- if (event->keyval == GDK_Tab) {
-    gint start,end;
-    if (event->state&GDK_CONTROL_MASK){
-	
-    	if (gtk_editable_get_selection_bounds(editable,&start,&end)){
-	/*printf("selection at %d %d\n",start,end);*/
-	  for (;start<=end;start++){
-		gchar *p;
-		p=gtk_editable_get_chars (editable,start,end);
-		/*printf("start=%d got char %s\n",start,p);*/
-		if (*p =='/') {
-			gtk_editable_delete_text (editable,start,end);
-			g_free(p);
-			break;
-		}
-		g_free(p);
-	  }
-   	  gtk_editable_select_region (editable, 0,0);
-    	  gtk_editable_set_position(editable,-1);
-	  return(TRUE);
-	  
-	}
-        return TRUE;
-    }
-    else
-    {
-    	fulltext=gtk_editable_get_chars (editable,0,-1);
-        for (;tmp;tmp=tmp->next){
-	   gchar *p=(gchar *)tmp->data;
-	   if (strcmp(p,fulltext)==0){
-		   tmp=tmp->next;
-		   break;
-	   }
-	}
-	g_free(fulltext);
-    	fulltext=gtk_editable_get_chars (editable,0,pos);
-    }
- }
-
-
- /* construct search string... (limited to 128 bytes)*/
-
- /* control-delete will remove the selected item from the
-  * history dbh file (to remove stale history items) */
-
- else if (event->keyval == GDK_BackSpace ||
-     event->keyval == GDK_Delete ||
-     event->keyval == GDK_KP_Delete ||
-     (event->keyval >= GDK_KP_0 && event->keyval <= GDK_KP_9) ||
-     (event->keyval >= GDK_space && event->keyval <= GDK_asciitilde)) {
-	/* get entry text */
-
-   	if (event->keyval == GDK_BackSpace ){
-      	  text[0]=gtk_editable_get_chars (editable,0,pos-1);
-    	  text[1]=gtk_editable_get_chars (editable,pos,-1);
-	  if (preselection) {
-		  g_free(fulltext);
-		  fulltext=NULL;
-	  }
-	  else fulltext=g_strconcat(text[0],text[1],NULL);
-	} 
-	else if (event->keyval == GDK_Delete ||
-     	    event->keyval == GDK_KP_Delete){
-	  if (active_dbh_file && event->state&GDK_CONTROL_MASK){ /* remove stale entries */
-    	    fulltext=gtk_editable_get_chars (editable,0,-1);
-	    remove_from_history(active_dbh_file,fulltext);
-	    print_diagnostics(treeview,"xf_INFO_ICON",
-			    _("Variable")," \"",fulltext,"\" ",
-			    _("Cancelled"),"\n",NULL);
-	    g_free(fulltext);
-	    cancel_input(treeview);
-	    return TRUE;
-	  } else {
-      	    text[0]=gtk_editable_get_chars (editable,0,pos);
-    	    text[1]=gtk_editable_get_chars (editable,pos+1,-1);
-	    if (preselection) {
-		  g_free(fulltext);
-		  fulltext=NULL;
-	    }
-	    else fulltext=g_strconcat(text[0],text[1],NULL);
-	  }
-	} else {
-      	  text[0]=gtk_editable_get_chars (editable,0,pos);
-    	  text[1]=gtk_editable_get_chars (editable,pos+1,-1);
-	  *c=event->keyval;	
-	  if (preselection) fulltext=g_strconcat(text[0],c,NULL);
-	  else fulltext=g_strconcat(text[0],c,text[1],NULL);
-	}		
-    	g_free(text[0]);
-    	g_free(text[1]);
- }
-	 
- if (fulltext){
-    /* look for in ordered GList*/
-    /*printf("DBG:fulltext is %s\n",fulltext);*/
-    for (;tmp;tmp=tmp->next){
-	gchar *p=(gchar *)tmp->data;
-	if (!p) continue;
-	if (strncmp(fulltext,p,strlen(fulltext))==0){
-	    gchar *to_write=p+strlen(fulltext);
-	    /*printf("DBG: gotcha %s->%s, to write=%s\n",fulltext,p,to_write);*/
-            /* if found, complete with untyped part selected.*/
-	    gtk_editable_delete_text (editable,0,-1);
-	    pos=0;
-	    gtk_editable_insert_text (editable,(const gchar *)fulltext,
-			     strlen(fulltext),&pos);
-	    pos2=pos;
-	    gtk_editable_insert_text (editable,(const gchar *)to_write,
-			     strlen(to_write),&pos2);
-    	    gtk_editable_select_region (GTK_EDITABLE (entry), pos, -1);
-    	    g_free(fulltext);
-
-	    return TRUE;
-	}
-    }
-    g_free(fulltext);
- }
- if (event->keyval == GDK_Tab) return TRUE;
-			    
- return (FALSE);
+ return FALSE;
 }
 
 
@@ -301,12 +132,17 @@ void show_input(GtkTreeView * treeview, int which)
     GtkCheckButton *check_button1, *check_button2;
     GtkLabel *label;
     GtkEntry *entry;
-    GtkWidget *combo;
     /*GtkToggleButton *w; */
     GtkTreeIter iter;
     char *path, *name;
     tree_entry_t *en;
     gchar *sname;
+    static gboolean first=TRUE;
+    static history_combo_info_t combo_info={
+	    NULL,NULL,NULL,NULL,NULL,NULL
+    };
+    static GList *emptylist=NULL;
+
 
     if(tree_details->tubo_object)
 	g_assert_not_reached();
@@ -317,15 +153,42 @@ void show_input(GtkTreeView * treeview, int which)
 
     label = (GtkLabel *) lookup_widget(tree_details->window, "input_label");
     entry = (GtkEntry *) lookup_widget(tree_details->window, "input_entry");
-    combo = lookup_widget(tree_details->window, "input_combo");
+    combo_info.treeview=treeview;
+    combo_info.entry=(GtkWidget *)entry;
+    combo_info.combo=lookup_widget(tree_details->window, "input_combo");
+    combo_info.active_dbh_file=NULL;
+    combo_info.list=emptylist;
+    
     hideit(tree_details->window, "filter_box");
     showit(tree_details->window, "input_box");
     showit(tree_details->window, "input_combo");
-    /* tie escape to cancel: */
-    g_signal_connect(G_OBJECT(combo), "key_press_event", 
+
+    if (first){
+	    first=FALSE;
+	    if (!emptylist){
+		    combo_info.list=emptylist=g_list_append(emptylist,"");
+	    }
+	    /* the double signal is to tie escape, which is particular to each combo */
+	    g_signal_connect(G_OBJECT(entry), "key_press_event", 
 		    G_CALLBACK(on_key_press), (gpointer) treeview);
-
-
+	    g_signal_connect(G_OBJECT(entry), "key_press_event", 
+		    G_CALLBACK(on_key_press_history), (gpointer) &combo_info);
+#if 0
+	    /* this here does not work because the signal should be set for
+	     * the combo window, not the combo. We should receive the key
+	     * press signals for the combo window and pass them to the
+	     * entry key handler so that they will get displayed. A tough point
+	     * is on how to deactivate the combo before the list is 
+	     * recalculated and activated again...*/
+	    g_signal_connect(G_OBJECT(combo_info.combo), "key_press_event", 
+		    G_CALLBACK(on_key_press_history), (gpointer) &combo_info);
+#endif
+  	    g_signal_connect (G_OBJECT(entry), "activate",
+                    G_CALLBACK (on_input_activate),
+                    (gpointer) &combo_info);
+    }
+    gtk_combo_set_popdown_strings(GTK_COMBO(combo_info.combo),emptylist);
+    set_limited_combo(&combo_info,NULL);
     hideit(tree_details->window, "checkbutton1");
     hideit(tree_details->window, "checkbutton2");
 
@@ -369,6 +232,11 @@ void show_input(GtkTreeView * treeview, int which)
 
     switch (which)
     {
+	case SAVE_BOOK_INPUT:
+	    gtk_entry_set_text(entry, "");
+	    gtk_label_set_text(label, _("Book"));
+	    PRINT_STATUS_INPUT;
+	    break;
 	case OPEN_BOOK_INPUT: 
 	{	
 	    set_book_combo(treeview);
@@ -383,19 +251,11 @@ void show_input(GtkTreeView * treeview, int which)
 	    PRINT_STATUS_INPUT;
 	    break;
 	}
-	case WORKDIR_INPUT: 
-	{	
-	    list=set_workdir_combo(treeview);
-	    gtk_label_set_text(label,workdir?_(workdir):_("Output dir"));
-    	    g_signal_connect(G_OBJECT(entry), "key_press_event", 
-		    G_CALLBACK(on_key_press2), (gpointer) treeview);
-	    PRINT_STATUS_INPUT;
-	    break;
-	}
 	case SYMLINK_INPUT:
 	    sname=g_strconcat(name,"-lnk",NULL);
 	    gtk_entry_set_text(entry, sname);
 	    g_free(sname);
+	    sname=NULL;
 	    gtk_label_set_text(label, _("Name"));
 	    PRINT_STATUS_INPUT;
 	    break;
@@ -409,7 +269,8 @@ void show_input(GtkTreeView * treeview, int which)
 	case SCRAMBLE_INPUT:
 	    sname=g_strconcat(_("Confirm"), " ",FILENAME(en),NULL);
 	    gtk_label_set_text(label, sname);
-	    g_free(sname);    
+	    g_free(sname); 
+	    sname=NULL;
 	    hideit(tree_details->window, "input_combo");
 	    print_status(treeview, "xf_QUESTION_ICON", 
 			    _("Scramble"),
@@ -418,7 +279,8 @@ void show_input(GtkTreeView * treeview, int which)
 	case UNSCRAMBLE_INPUT:
 	    sname=g_strconcat(_("Confirm"), " ",FILENAME(en),NULL);
 	    gtk_label_set_text(label, sname);
-	    g_free(sname);    
+	    g_free(sname);  
+	    sname=NULL;
 	    hideit(tree_details->window, "input_combo");
 	    print_status(treeview, "xf_QUESTION_ICON", 
 			    _("Unscramble"), NULL);
@@ -433,44 +295,39 @@ void show_input(GtkTreeView * treeview, int which)
 	    gtk_label_set_text(label, _("Name"));
 	    PRINT_STATUS_INPUT;
 	    break;
-
-
 	case NEW_INPUT:
 	    set_new_combo(treeview);
 	    gtk_label_set_text(label, _("Name"));
 	    print_status(treeview, "xf_QUESTION_ICON", _("Name"), NULL);
 	    gtk_button_set_label((GtkButton *) check_button1, _("Directory"));
 	    showit(tree_details->window, "checkbutton1");
-
-
+	    break;
+	case WORKDIR_INPUT: 
+	    set_workdir_combo(&combo_info);	
+	    gtk_label_set_text(label,workdir?_(workdir):_("Output dir"));
+	    PRINT_STATUS_INPUT;
 	    break;
 	case GOTO_INPUT:
-	    list=set_goto_combo(treeview);
+	    set_goto_combo(&combo_info);
 	    gtk_label_set_text(label, _("Go to"));
 	    PRINT_STATUS_INPUT;
-    	    g_signal_connect(G_OBJECT(entry), "key_press_event", 
-		    G_CALLBACK(on_key_press2), (gpointer) treeview);
 	    break;
 	case RUN_INPUT:
 	case RUN_DOUBLE_CLICK:
 	    switch (which)
 	    {
 		case RUN_INPUT:
-		    list=set_run_combo(treeview);
+		    set_run_combo(&combo_info);
 		    gtk_label_set_text(label, _("Run"));
 		    PRINT_STATUS_INPUT;
-    	    g_signal_connect(G_OBJECT(entry), "key_press_event", 
-		    G_CALLBACK(on_key_press2), (gpointer) treeview);
 		    break;
 		case RUN_DOUBLE_CLICK:
-		    list=set_run_combo(treeview);
+		    set_run_combo(&combo_info);
 		    gtk_label_set_text(label, _("Open with"));
 		    gtk_button_set_label((GtkButton *) check_button2, 
 				    _("Remember"));
 		    showit(tree_details->window, "checkbutton2");
 		    PRINT_STATUS_INPUT;
-    	    g_signal_connect(G_OBJECT(entry), "key_press_event", 
-		    G_CALLBACK(on_key_press2), (gpointer) treeview);
 		    break;
 
 	    }
@@ -483,12 +340,13 @@ void show_input(GtkTreeView * treeview, int which)
 	    break;
     }
     g_free(path);
+    path=NULL;
     gtk_widget_grab_focus((GtkWidget *) entry);
     /* these will preselect */
     
     if (which == NEW_INPUT || which == SYMLINK_INPUT ||
 	which ==  RUN_INPUT || which == RUN_DOUBLE_CLICK ||
-        which == PRINT_INPUT){
+        which == PRINT_INPUT || which == SAVE_BOOK_INPUT){
       gtk_editable_set_position(GTK_EDITABLE (entry),0);
       gtk_editable_select_region (GTK_EDITABLE (entry), 0, -1);
     } else {
@@ -531,13 +389,41 @@ void ok_input(GtkTreeView * treeview)
 	    g_free(bookfile);
 	    bookfile=g_strdup(choice);
     	    g_free(choice);
+	    choice=NULL;
     	    reload_book(treeview); 
 	    return;
+	case SAVE_BOOK_INPUT:
+	    {
+	      gchar *oldname,*newname;
+	      int status;
+	      if (!choice || !strlen(choice)) return;
+	      oldname=g_strdup(get_bookfile_path());
+	      g_free(bookfile);
+	      bookfile=g_strdup(choice);
+    	      g_free(choice);
+	      choice=NULL;
+	      newname=g_strdup(get_bookfile_path());
+	    
+	      /*printf("DBG:replacing %s for %s\n",oldname,newname);*/
+	      /* XXX: probably should warn on overwritting */
+	      if (fork()) wait(&status);
+	      else {
+		      execlp("cp","cp","-f",oldname,newname,0);
+		      printf("DBG: should not be reached. fork failed!\n");
+	      }
+	      g_free(oldname);
+	      oldname=NULL;
+	      g_free(newname);
+	      newname=NULL;
+    	      reload_book(treeview); 
+	    }
+	    return;
 	case WORKDIR_INPUT:
 	    g_free(workdir);
 	    workdir=g_strdup(choice);
 	    save_workdir_history(workdir);
     	    g_free(choice);
+	    choice=NULL;
 	    /* OTHER_INPUT should cancel whatever is waiting on gtk loop */
     	    /*tree_details->input = OTHER_INPUT;*/
 	    gtk_main_quit();
@@ -628,6 +514,7 @@ void ok_input(GtkTreeView * treeview)
 				    in_cmd, NULL);
 		}
 		g_free(in_cmd);
+		in_cmd=NULL;
 	    }
 	    break;
 	case GOTO_INPUT:
@@ -702,7 +589,9 @@ void ok_input(GtkTreeView * treeview)
 		    }
 		    new_it(treeview, path, name, gtk_toggle_button_get_active((GtkToggleButton *) w));
 		    g_free(path);
+		    path=NULL;
 		    g_free(name);
+		    name=NULL;
 		}
 		if(set_load_wait(&tree_details))
 		{
@@ -715,14 +604,18 @@ void ok_input(GtkTreeView * treeview)
 	    break;
     }
     g_free(nfile);
+    nfile=NULL;
     g_free(choice);
+    choice=NULL;
     tree_details->input = OTHER_INPUT;
     local_monitor(treeview,TRUE);
 }
 
 void cancel_input(GtkTreeView * treeview)
 {
-    tree_details_t *tree_details = get_tree_details(treeview);
+    tree_details_t *tree_details;
+    if (!treeview) return;
+    tree_details = get_tree_details(treeview);
     hideit(tree_details->window, "input_box");
     if(tree_details->preferences & SHOW_F)
 	showit(tree_details->window, "filter_box");
@@ -751,7 +644,6 @@ void on_input_ok(GtkButton * button, gpointer user_data)
 {
     GtkTreeView *treeview = get_treeview((GtkWidget *) button);
     ok_input(treeview);
-
 }
 
 
