@@ -70,8 +70,8 @@ extern gchar *bookfile;
  * then  "$command $filename" is used */
 autotype_t autotype_dir[]= {
 	/* archive file extractions (queued) */
-	{".tar.gz","tar -czf ","tar -czf",N_("Output dir"),1},
-	{".iso","mkisofs -dDlLJNRTUP Created_by_XFce4_desktop -o ","mkisofs",N_("Output dir"),1},
+	{".tar.gz","tar -czf ",N_("Create tarball"),N_("Output dir"),1},
+	{".iso","mkisofs -dDlLJNRTUP Created_by_XFce4_desktop -o ",N_("Create isofs"),N_("Output dir"),1},
 	{NULL,NULL}
 };
 #endif
@@ -79,7 +79,9 @@ autotype_t autotype_dir[]= {
 autotype_t autotype[]= {
 	/* archive file extractions (queued) */
 	{".tar.gz","tar -xzf ",N_("Extract files from"),N_("Extract into"),1},
+	{".tar.gz","tar -tzf ","Tarball contents",NULL,1}, /* XXX remove when tarball expansion enabled */
 	{".gz","gunzip",N_("Uncompress"),NULL,1},
+	{".tar","tar -tf ","Tarball contents",NULL,1},/* XXX remove when tarball expansion enabled */
 	{".tar","tar -xf ",N_("Extract files from"),N_("Extract into"),1},
 	{".tar.bz2","tar --use-compress-program bunzip2 -xf",N_("Extract files from"),N_("Extract into"),1},
 	{".bz2","bunzip2",N_("Uncompress"),NULL,1},
@@ -88,8 +90,9 @@ autotype_t autotype[]= {
 	{".zip","unzip",N_("Uncompress"),NULL,1},
 	
 	{".tgz","tar -xzf ",N_("Extract files from"),N_("Extract into"),1},
+	{".tgz","tar -tzf ","Tarball contents",NULL,1}, /* XXX remove when tarball expansion enabled */
 #if 1
-	{".iso","burncd -f /dev/acd0c -s max data ","burncd",NULL,1},
+	{".iso","burncd -f /dev/acd0c -s max data \%s fixate","burncd",NULL,1},
 	{".iso","cdrecord -v speed=24 dev=0,0 ","cdrecord",NULL,1},
 #endif
 	
@@ -129,40 +132,73 @@ autotype_t autotype[]= {
 	{".deb","dpkg -i ",N_("Install package"),NULL,1},
 #endif
 	/* assorted packages (not queued) */
-	{".lyx","lyx","LyX",NULL,0},
+#if 0
+	/* some of these are now the application mime types 
+	 * remove them from xffm 4.1 when the duplicate types
+	 * is introduced into the applications.xml file */
+	{".lyx","lyx","lyx",NULL,0},
 	{".ps","gv","gv",NULL,0},
 	{".pdf","gv","gv",NULL,0},
-	{".abw","abiword","AbiWord",NULL,0},
-	{".rtf","abiword","AbiWord",NULL,0},
-	{".rtf","oowriter","Open office",NULL,0},
-	{".dvi","xdvi","Xdvi",NULL,0},
-	{".html","mozilla","Mozilla",NULL,0},
-	{".html","dillo","Dillo",NULL,0},
-	{".html","netscape","Netscape",NULL,0},
-	{".sdw","oowriter","Open office",NULL,0},
-	{".doc","abiword","AbiWord",NULL,0},
-	{".doc","oowriter","Open Office",NULL,0},
-	{".log","gvim -rv ","Gvim",NULL,0},
-	{".tex","gvim","Gvim",NULL,0},
-	{".txt","abiword","AbiWord",NULL,0},
-	{".txt","oowriter","Open Office",NULL,0},
-	{".txt","emacs","Emacs",NULL,0},
-	{".txt","gvim","Gvim",NULL,0},
-	{".sdc","oocalc","Open office",NULL,0},
-	{".sxc","oocalc","Open office",NULL,0},
-	{".sh","gvim","Gvim",NULL,0},
-	{".py","gvim","Gvim",NULL,0},
-	{".bash","gvim","Gvim",NULL,0},
-	{".csh","gvim","Gvim",NULL,0},
-	{".pl","gvim","Gvim",NULL,0},
-	{".wav","soxplay","Soxplay",NULL,0},
-	{".gif","gimp","The Gimp",NULL,0},
-	{".tiff","gimp","The Gimp",NULL,0},
-	{".tif","gimp","The Gimp",NULL,0},
-	{".jpg","gimp","The Gimp",NULL,0},
-	{".jpeg","gimp","The Gimp",NULL,0},
-	{".png","gimp","The Gimp",NULL,0},
-	{".xpm","gimp","The Gimp",NULL,0},
+	{".dvi","xdvi","xdvi",NULL,0},
+	{".sdw","oowriter","oowriter",NULL,0},
+	{".sdw","swriter","swriter",NULL,0},
+	{".html","dillo","dillo",NULL,0},
+	{".abw","abiword","abiword",NULL,0},
+	{".bmp","display","display",NULL,0},
+	{".eps","display","display",NULL,0},
+	{".gif","display","display",NULL,0},
+	{".tiff","display","display",NULL,0},
+	{".tif","display","display",NULL,0},
+	{".jpg","display","display",NULL,0},
+	{".jpeg","display","display",NULL,0},
+	{".png","display","display",NULL,0},
+	{".xpm","display","display",NULL,0},
+	{".txt","gvim","gvim",NULL,0},
+	{".sdc","oocalc","oocalc",NULL,0},
+	{".sxc","oocalc","oocalc",NULL,0},
+	{".sdc","scalc","scalc",NULL,0},
+	{".sxc","scalc","scalc",NULL,0},
+	{".wav","soxplay","soxplay",NULL,0},
+
+	
+/* extras entries in menu:
+ * for 4.1 these should be added to the mime application file
+ * and processed into xffm from there... (allowing
+ * repeat entries from the mime application file)
+ * */
+	{".wav","soxplay","soxplay",NULL,0},
+	{".tex","abiword","abiword",NULL,0},
+	{".tex","gvim","gvim",NULL,0},
+	{".doc","abiword","abiword",NULL,0},
+	{".doc","oowriter","oowriter",NULL,0},
+	{".doc","swriter","swriter",NULL,0},
+	{".dvi","xterm -e dvips ","dvips",NULL,0},
+	{".pdf","xterm -e pdf2ps ","pdf2ps",NULL,0},
+	{".rtf","abiword","abiword",NULL,0},
+	{".rtf","oowriter","oowriter",NULL,0},
+	{".rtf","swriter","swriter",NULL,0},
+	{".html","dillo","dillo",NULL,0},
+	{".html","mozilla","mozilla",NULL,0},
+	{".html","netscape","netscape",NULL,0},
+	{".log","gvim","gvim",NULL,0},
+	{".txt","abiword","abiword",NULL,0},
+	{".txt","oowriter","oowriter",NULL,0},
+	{".txt","swriter","swriter",NULL,0},
+	{".txt","emacs","emacs",NULL,0},
+	{".sh","gvim","gvim",NULL,0},
+	{".py","gvim","gvim",NULL,0},
+	{".bash","gvim","gvim",NULL,0},
+	{".csh","gvim","gvim",NULL,0},
+	{".pl","gvim","gvim",NULL,0},
+
+	{".gif","gimp","gimp",NULL,0},
+	{".tiff","gimp","gimp",NULL,0},
+	{".tif","gimp","gimp",NULL,0},
+	{".jpg","gimp","gimp",NULL,0},
+	{".jpeg","gimp","gimp",NULL,0},
+	{".png","gimp","gimp",NULL,0},
+	{".xpm","gimp","gimp",NULL,0},
+#endif
 	/*{"","","",NULL},*/
 	{NULL,NULL}
 };
@@ -266,12 +302,12 @@ static void check_select(GtkTreeModel * model, GtkTreePath * path, GtkTreeIter *
    	   if (!remote_location)remote_location=location;
 	   else {
 	     if (strcmp(remote_location,location)!=0)invalid_input=TRUE;
-	     g_free(location);
+	     g_free(location);location=NULL;
 	   }
    	   if (!remote_pass)remote_pass=server_pass;
 	   else {
 	     if (strcmp(remote_pass,server_pass)!=0)invalid_input=TRUE;
-	     g_free(server_pass);
+	     g_free(server_pass);server_pass=NULL;
 	   }
 	}
     }
@@ -293,6 +329,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
     GtkTreeModel *treemodel = gtk_tree_view_get_model(treeview);
     GtkTreeSelection *selection = gtk_tree_view_get_selection(treeview);
     tree_details_t *tree_details = (tree_details_t *) get_tree_details(treeview);
+    gchar *cmd=NULL,*text=NULL;
 
 
     if(tree_details->loading)
@@ -352,7 +389,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 	print_status_tmp(treeview, resolve_icon_small(en),  FILENAME(en), NULL);
 	if(IS_PATH(en->type)){
 	  turn_on(treeview);
-          showit(popup, "show_hidden1");
+          if (!(tree_details->preferences & SHOW_DOT)) showit(popup, "show_hidden1");
 	  showit(popup, "tools1");	
 	  showit(popup, "separator25");
 	  showit(popup, "separator13");
@@ -410,7 +447,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 		showit(popup, "refresh3"); /* for netthings in bookmarks */
 		showit(popup, "copy1");
 		if (IS_NETDIR(en->subtype)||IS_XF_NETSHARE(en->subtype)) {
-		    showit(popup, "show_hidden1");
+		    if (!(tree_details->preferences & SHOW_DOT)) showit(popup, "show_hidden1");
 		    showit(popup, "separator25");
 		    showit(popup, "separator13");
 		    showit(popup, "select_all2");
@@ -421,7 +458,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 	    }
 	    if(IS_PATH(en->type))
 	    {
-        	gchar *cmd;
+		int j;
       		reg_t *prg = reg_prog_by_file (en->path);
 		showit(popup, "file1");
 		showit(popup, "remove1");
@@ -446,7 +483,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 		  cmd=g_strconcat(_("Execute")," ",
 					  FILENAME(en),NULL);
 		  gtk_label_set_text((GtkLabel *)label,my_utf_string(cmd));
-		  g_free(cmd);
+		  g_free(cmd);cmd=NULL;
  		  showit(popup,"autotype_Prun");
 		}
 		showit(popup, "separator25");
@@ -457,7 +494,10 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 		showit(popup, "paste1");
 		hideit(popup, "scramble1");
 		hideit(popup, "unscramble1");
-      		if (prg) {/* look in registered programs first */
+      		if (prg && strlen(prg->app)) {
+			/* look in registered programs first */
+			/* FIXME: should check whether prg->app can be run
+			 * before generating the menu entry... */
 		  GtkWidget *label,*a=lookup_widget(popup,"autotype_R");
     		  label = gtk_bin_get_child(GTK_BIN(a));
 		  if (prg->arg) {
@@ -468,37 +508,70 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 			  cmd=g_strconcat(prg->app, NULL);
 					  /*" ",FILENAME(en),NULL);*/
 		  }
-		  gtk_label_set_text((GtkLabel *)label,my_utf_string(cmd));
-		  g_free(cmd);
+		  text=g_strconcat(cmd," \'",FILENAME(en),"\'",NULL);
+		  gtk_label_set_text((GtkLabel *)label,my_utf_string(text));
+		  g_free(text);text=NULL;
 		  showit(popup, "autotype_R");
-      	  	} 
+      	  	}
+
 		{/* autotypes */	      
       		   char *loc;
-		   int i,j;
+		   int i;
 		   GtkWidget *label;
 		   for (j=i=0;auto_C_name[j]&&autotype[i].extension;i++){
 		       GtkWidget *a=lookup_widget(popup,auto_C_name[j]);
 		       loc=strstr(en->path,autotype[i].extension);
 		       if ((loc)&&(strcmp(loc,autotype[i].extension)==0)){
-		           gchar *text,*prg;
+		           gchar *prog;
     		  	   label = gtk_bin_get_child(GTK_BIN(a));
 		           text=g_strconcat(_(autotype[i].label),
-					   " ", FILENAME(en),
+					   " \"", FILENAME(en),"\"",
 					   NULL);
 			   gtk_label_set_text((GtkLabel *)label,
 					   my_utf_string(text));
 			   g_free(text); text=NULL;
-			   prg=g_strdup(autotype[i].command);
-			   if (strstr(prg," ")) prg=strtok(prg," ");
-			   text=g_find_program_in_path(prg);
-		  	   if (text) showit(popup, auto_C_name[j]);
-			   else g_free(text);
-			   g_free(prg);
+			   
+			   prog=g_strdup(autotype[i].command);
+			   if (strstr(prog," ")) prog=strtok(prog," ");
+			   text=g_find_program_in_path(prog);
+					   
+		  	   if (text && !(cmd && strcmp(prog,cmd)==0)) {
+				   showit(popup, auto_C_name[j]);
+			   }
+			   			     
+			   g_free(text);text=NULL;
+			   g_free(prog);prog=NULL;
 			   j++;
 		       }
 		   }
 		}
-
+	        /* duplicate mime types */
+		{
+		  GtkWidget *label;
+		  int i;
+		  const char *prog;
+		  for (i=0; j<10 && i<10;i++){
+			  /*printf("DBG: duplicate slot %d\n",j);*/
+		       	  GtkWidget *a=lookup_widget(popup,auto_C_name[j]);
+			  prog=reg_duplicate_app(en->path,i);
+			  if (!prog) {
+				  /*printf("DBG: no duplicate!\n");*/
+				  break;
+			  }
+			  /*printf("DBG: duplicate FOUND!\n");*/
+    		  	  label = gtk_bin_get_child(GTK_BIN(a));
+		          text=g_strconcat(prog," \"", FILENAME(en),"\"",NULL);
+			  gtk_label_set_text((GtkLabel *)label,my_utf_string(text));
+				   
+		  	  if (!(cmd && strcmp(prog,cmd)==0)) {
+				   showit(popup, auto_C_name[j]);
+			  } 				   
+			  j++;   
+			  g_free(text);text=NULL;
+		  } 
+		}
+		g_free(cmd);cmd=NULL;		   
+		
 		if(IS_TRASH_TYPE(en->type))
 		{
 		    showit(popup, "uncollect_from_trash");
@@ -506,8 +579,10 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 
 		if(IS_DIR(en->type))
 		{
-		    showit(popup, "preview_images1");
-		    if (IS_NETTHING(en->subtype) || HAS_HIDDEN(en->type)) 
+		    if (!(tree_details->preferences & IMAGE_PREVIEW)) 
+			    showit(popup, "preview_images1");
+		    if (IS_NETTHING(en->subtype) || HAS_HIDDEN(en->type)
+			&& !(tree_details->preferences & SHOW_DOT)) 
 			    showit(popup, "show_hidden1");
 		    showit(popup, "sort2");
 		    showit(popup, "collect_trash1");
@@ -526,10 +601,10 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 		   for (j=i=0;auto_D_name[j]&&autotype_dir[i].label;i++){
 		       GtkWidget *a=lookup_widget(popup,auto_D_name[j]);
 		       {
-		           gchar *text,*prg;
+		           gchar *prg;
     		  	   label = gtk_bin_get_child(GTK_BIN(a));
-		           text=g_strconcat(_("Run")," ",_(autotype_dir[i].label),
-					   " ", FILENAME(en),
+		           text=g_strconcat(_(autotype_dir[i].label),
+					   " \"", FILENAME(en),"\"",
 					   NULL);
 			   gtk_label_set_text((GtkLabel *)label,
 					   my_utf_string(text));
@@ -538,8 +613,8 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 			   if (strstr(prg," ")) prg=strtok(prg," ");
 			   text=g_find_program_in_path(prg);
 		  	   if (text) showit(popup, auto_D_name[j]);
-			   g_free(text);
-			   g_free(prg);
+			   g_free(text);text=NULL;
+			   g_free(prg);prg=NULL;
 			   j++;
 		       }
 		   }
@@ -548,7 +623,6 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 #if 0
 		    {
         		GtkWidget *label;
-        		gchar *text=NULL;
 			GtkWidget *a=lookup_widget(popup,"autotype_D");
     			label = gtk_bin_get_child(GTK_BIN(a));
 			//label=(GtkLabel *)(((GtkBin *)(a))->child);
@@ -630,6 +704,8 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 		showit(popup, "sort2");
 		showit(popup, "purge_bookmarks1");
 		showit(popup, "clear_all_bookmarks");
+		showit(popup, "save_book2");
+		showit(popup, "open_book2");
 	    }
 
 
@@ -655,6 +731,7 @@ void do_select_popup(GtkTreeView * treeview, GtkTreePath *selectpath,gint32 t)
 
 	gtk_menu_popup(GTK_MENU(popup), NULL, NULL, NULL, NULL, 3, t);
 			/*event->time);*/
+	
 	widget = lookup_widget(tree_details->window, "preview_images1");
 	gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(widget), 
 			SHOWS_IMAGES(en->type));
@@ -781,7 +858,7 @@ on_defaultbook_activate                (GtkMenuItem     *menuitem,
 {
     GtkTreeView *treeview = (GtkTreeView *) lookup_widget((GtkWidget *) menuitem, "treeview");
 	
-    g_free(bookfile);
+    g_free(bookfile);bookfile=NULL;
     bookfile=NULL;
     reload_book(treeview);
 }
@@ -832,7 +909,7 @@ on_rememberbook_activate               (GtkMenuItem     *menuitem,
               printf ("DBG:glob match: %s\n", g);
 	      
 #endif
-	      g_free(g);
+	      g_free(g);g=NULL;
 	   }
 	}
   	globfree (&dirlist);
