@@ -312,6 +312,7 @@ autofunction_workdir  (GtkTreeView *treeview, tree_entry_t *en,
     tree_details_t *tree_details = get_tree_details(treeview);
          
   g_free(workdir);
+  workdir=NULL;
   if (querypath){
       workdir=g_strdup(querypath); 
       /* remove any previous gtk loop: */ 
@@ -341,6 +342,16 @@ autofunction_workdir  (GtkTreeView *treeview, tree_entry_t *en,
   return TRUE;
 }
 
+/* autofunction for directories works different: 
+ * directories: the path argument is relative, so we must do a chdir
+ * 		to the path directory.
+ * 		The output specified will carry along the fullpath for 
+ * 		output. The workdir will be that where the directory 
+ * 		resides. We must do a chdir here.
+ * files:	the path argument is absolute. So we just do a chdir to
+ * 		the specified workdir before execution.
+ *
+ * 		*/
 static void
 autofunction  (GtkTreeView *treeview, tree_entry_t *en,
 		char **arguments,char *d,gboolean queued) 
@@ -382,7 +393,122 @@ autofunction  (GtkTreeView *treeview, tree_entry_t *en,
   return;
 }
 
+void
+on_autotype_C                          (GtkMenuItem     *menuitem,
+                                        gpointer         user_data)
+{
+  char *argv[MAX_ARGS];
+  char command[255];
+  int offset=(int)((long)user_data);
+  char *loc;
+  int i=0,j; 
+  gboolean bymime=FALSE;
+  tree_entry_t *en;
+  GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
+  GtkTreeIter iter;
+  static gchar *gs=NULL;
 
+  en = get_selected_entry(treeview, &iter);
+  if (gs) {g_free(gs); gs=NULL;}
+
+       
+  /*printf("DBG:offset=%d\n",offset);*/
+  if (!en || !en->path || !IS_PATH(en->type)) return;
+  if (offset >=10) return; /* only ten menu elements */
+  
+  for (i=0;autotype[i].extension;i++){
+       /*printf("dbg: autotype=%s,%s\n",autotype[i].extension,autotype[i].command);*/
+       loc=strstr(en->path,autotype[i].extension);
+       if ((loc)&&(strcmp(loc,autotype[i].extension)==0)) break;
+  }
+  
+  
+  command[0]=0;
+  for (j=0; j<=offset && autotype[i+j].command; j++);
+  /* printf("DBG:command=%s\n",autotype[i+j].command); */
+  
+
+  if (offset <= j && autotype[i+offset].command){
+      /* FIXME: do a check whether if the command is sudo allowed
+	       * without password, if so then: */
+      if(getenv("XFFM_USE_SUDO")&&strlen(getenv("XFFM_USE_SUDO"))&&
+	 (strncmp(command,"pkg_add",strlen("pkg_add"))==0 ||
+	  strncmp(command,"pkg_update",strlen("pkg_update"))==0 ||
+	  strncmp(command,"burncd",strlen("burncd"))==0 ||
+	  strncmp(command,"cdrecord",strlen("cdrecord"))==0 ||
+	  strncmp(command,"rpm",strlen("rpm"))==0 ||
+	  strncmp(command,"dpkg",strlen("dpkg"))==0 )
+	) {  
+         if (strlen(autotype[i+offset].command)>249) return;
+	 sprintf(command,"sudo %s",autotype[i+offset].command);  
+      } else {
+         if (strlen(autotype[i+offset].command)>255) return;
+         strcpy(command,autotype[i+offset].command);
+      }  
+  }
+  else {
+	/* must be a mime type */
+	GtkWidget *label;
+	const gchar *text;
+	label = gtk_bin_get_child(GTK_BIN(menuitem));
+	if (!label) return;
+	text=gtk_label_get_text ((GtkLabel *)label);
+	if (!text || strlen(text)==0) return;
+	if (strlen(text) > 255) return;
+	strncpy(command,text,255);
+	/* strip the filename */
+	if (strchr(command,'\"')) *strrchr(command,'\"')=0; /* last " */
+	if (strchr(command,'\"')) *strrchr(command,'\"')=0; /* first " */
+	*(strrchr(command,' '))=0;
+  	/*printf("DBG:command is =%s\n",text);*/
+	bymime=TRUE;
+  }
+  if (!strlen(command)) return;
+  
+   
+  
+  /* this function call will enter a gtk loop if querypath != NULL*/ 
+  if (!bymime) {
+	if (!autofunction_workdir (treeview, en,autotype[i+offset].querypath)) {
+	  return;
+	}
+	
+  }
+  i=parse_runline(argv,command, NULL,NULL);
+    /* command argv done, only need to add the file path */
+  for (j=1;argv[j];j++) {
+	    if (strcmp(argv[j],"\%s")==0){
+		    argv[j]=en->path;
+		    break;
+	    }
+	    if (strstr(argv[j],"\%s")){ /* XXX where is gs defined? are we leaking memory here? */
+		    gs=g_strconcat(argv[j]," ",en->path,NULL);
+		    g_snprintf(gs,strlen(gs),argv[j],en->path);
+		    argv[j]=gs;
+		    break;
+	    }
+    }
+  
+  if (!argv[j]) {
+    argv[i++]=en->path;
+    argv[i] = NULL;
+  } 
+	 
+#ifdef DEBUG
+  printf("DBG:");for (j=0;argv[j];j++) printf("%s ",argv[j]);printf("\n");
+#endif
+
+  if (!bymime && autotype[i+j].command && autotype[i+offset].queued)
+	  autofunction(treeview,en,argv,workdir, autotype[i+offset].queued);
+  else {
+	  runv(treeview,argv);
+  }
+
+  return;
+}
+
+
+#if 0
 void
 on_autotype_C                          (GtkMenuItem     *menuitem,
                                         gpointer         user_data)
@@ -392,6 +518,7 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
   int offset=(int)((long)user_data);
   char *loc;
   int i=0,j; 
+  gboolean bymime=FALSE;
   tree_entry_t *en;
     GtkTreeView *treeview = get_treeview((GtkWidget *) menuitem);
     GtkTreeIter iter;
@@ -400,6 +527,7 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
        
   /*printf("DBG:offset=%d\n",offset);*/
   if (!en || !en->path || !IS_PATH(en->type)) return;
+  if (offset >=10) return; /* only ten menu elements */
   
   for (i=0;autotype[i].extension;i++){
        /*printf("dbg: autotype=%s,%s\n",
@@ -407,14 +535,29 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
        loc=strstr(en->path,autotype[i].extension);
        if ((loc)&&(strcmp(loc,autotype[i].extension)==0)) break;
   }
+  
+  command[0]=0;
   for (j=0;j<=offset;j++){
   	/*printf("DBG:command=%s\n",autotype[i+j].command);*/
-  	if (autotype[i+j].command==NULL) return;
-  }
+  	if (autotype[i+j].command==NULL || autotype[i+j].extension==NULL
+			|| !loc || strcmp(loc,autotype[i+j].extension)!=0 ) {
 
-  
+		GtkWidget *label;
+      		const gchar *text;
+    		label = gtk_bin_get_child(GTK_BIN(menuitem));
+		if (!label) return;
+		text=gtk_label_get_text ((GtkLabel *)label);
+		if (!text || strlen(text)==0) return;
+		strncpy(command,text,255);
+  	/*printf("DBG:command is =%s\n",text);*/
+		bymime=TRUE;
+		break;
+		/*return;*/
+	}
+  }
   j=0;
-  if (strstr(autotype[i+offset].command," ")){
+
+  if (command[0]==0 && autotype[i+offset].command) {
       if (strlen(autotype[i+offset].command)>254) return;
       strcpy(command,autotype[i+offset].command);
       /* sudo-izable stuff */
@@ -431,7 +574,11 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
 	      argv[j++]="sudo";
 	      /* when check is implemented, add env var to mcs manager
 	       * (put check also in fstab.c) */
-      }	      
+      }	     
+   } 
+   if (command[0]==0) return;
+   
+   if (strstr(command," ")){
       argv[j++]=strtok(command," ");
       do {
 	      argv[j]=strtok(NULL," ");
@@ -439,25 +586,33 @@ on_autotype_C                          (GtkMenuItem     *menuitem,
 	      j++;
 	      if (j>=64) { argv[63]=0; break; }
       } while (1);
-  } else argv[j++]=autotype[i+offset].command;
-  argv[j++]=en->path;
+   } 
+  
+  if (bymime) argv[j-1]=en->path;
+  else argv[j++]=en->path;
+  
   if (strcmp(argv[0],"burncd")==0 || strcmp(argv[1],"burncd")==0) 
 	  argv[j++]="fixate";
   argv[j]=0;
 
-  /* this function call will enter a gtk loop */ 
-  if (!autofunction_workdir (treeview, en,autotype[i+offset].querypath)) {
+  /* this function call will enter a gtk loop if querypath != NULL*/ 
+  if (!bymime) {
+	if (!autofunction_workdir (treeview, en,autotype[i+offset].querypath)) {
 	  return;
+	}
   }
+  
   /*printf("DBG:");for (j=0;argv[j];j++) printf("%s ",argv[j]);printf("\n");*/
-  if (autotype[i+offset].queued)
+  if (!bymime && autotype[i+j].command && autotype[i+offset].queued)
 	  autofunction(treeview,en,argv,workdir, autotype[i+offset].queued);
   else runv(treeview,argv);
   return;
 }
+#endif
 
 
-
+/* in this function, the first argument we add will be the output file.
+ * it should be absolute, determined by the result of workdir. */
 
 void
 on_autotype_D                          (GtkMenuItem     *menuitem,
@@ -511,25 +666,42 @@ on_autotype_D                          (GtkMenuItem     *menuitem,
 	      if (j>=64) { argv[63]=0; break; }
       } while (1);
   } else argv[j++]=autotype_dir[i+offset].command;
-  if (autotype_dir[i+offset].extension) {
-	  if (strlen(FILENAME(en))+strlen(autotype_dir[i+offset].extension) >254) 
-		  return;
-	  sprintf(output,"%s%s",FILENAME(en),autotype_dir[i+offset].extension);
-	  argv[j++]=output;
-  }
-  argv[j++]=en->path;
-  argv[j]=0;
 
+  /* get the output directory */
   /* this function call will enter a gtk loop */ 
   if (!autofunction_workdir (treeview, en,autotype_dir[i+offset].querypath)) {
 	  return;
   }
+
+  if (autotype_dir[i+offset].extension) {
+	  if (strlen(workdir)+strlen(FILENAME(en))+strlen(autotype_dir[i+offset].extension) >253) 
+		  return;
+	  sprintf(output,"%s/%s%s",workdir,FILENAME(en),autotype_dir[i+offset].extension);
+	  argv[j++]=output;
+  }
+  
+  /* now set the workdir (no internal gtk loop) */
+  if (!autofunction_workdir (treeview, en,NULL)) return;
+  
+  /* for the path, since g_get_basename has to be freed, it will not work with queued commands */
+  if (!strrchr(en->path,'/')){
+	  printf("DBG: no separator in path!\n");
+	  return;
+  }
+
+  if (!strlen(strrchr(en->path,'/')+1)) argv[j++]=strrchr(en->path,'/');
+  else argv[j++]=strrchr(en->path,'/')+1;
+
+  argv[j]=0;
+
+  
   /*printf("DBG:");for (j=0;argv[j];j++) printf("%s ",argv[j]);printf("\n");*/
   if (autotype_dir[i+offset].queued)
 	  autofunction(treeview,en,argv,workdir, autotype_dir[i+offset].queued);
   else runv(treeview,argv);
   return;
 }
+
 #if 0
 {
   char *arguments[5];
