@@ -1,3 +1,9 @@
+gboolean is_in_history(char *dbh_file,char *path2save);
+gint on_key_press_history(GtkWidget * entry, GdkEventKey * event, gpointer data);
+void count_selection(GtkTreeModel * treemodel, 
+		GtkTreePath * treepath, 
+		GtkTreeIter * iter, 
+		gpointer data);
 void get_history_list(GList **in_list,char *dbh_file,char *top);
 void save_to_history(char *dbh_file,char *path2save);
 void remove_from_history(char *dbh_file,char *path2save);
@@ -5,7 +11,6 @@ void remove_from_history(char *dbh_file,char *path2save);
 gchar *utf_2_local_string(char *t);
 void process_pending_gtk();
 gboolean find_root(GtkTreeView * treeview, int which);
-GtkTreeView *get_selected_treeview(GtkWidget *w);
 const gchar *get_xffm_home(void);
 void disable_diagnostics(void);
 void enable_diagnostics(void);
@@ -56,6 +61,7 @@ tree_details_t *get_tree_details(GtkTreeView * treeview);
 void set_font(GtkTreeView * treeview, GtkTreeIter * iterator);
 
 GtkTreeView *get_treeview(GtkWidget * widget);
+GtkTreeView *get_selected_treeview(GtkWidget * widget);
 void hideit(GtkWidget * parent, gchar * widget_name);
 void showit(GtkWidget * parent, gchar * widget_name);
 void hide_text(GtkWidget * parent);
