@@ -61,9 +61,7 @@
 #include <gtk/gtk.h>
 #include <glib.h>
 
-#ifdef XFCE_SUPPORT
 #include <libxfce4util/util.h>
-#endif
 
 #include "glade_gui.h"
 #include "glade_support.h"
@@ -132,6 +130,8 @@ int runv(GtkTreeView * treeview, char *argv[])
     GThread *thread;
 #endif
     gchar *fullpath;
+    const char *fork_dir;
+    
 
     fullpath=g_find_program_in_path(argv[0]);
 
@@ -146,9 +146,18 @@ int runv(GtkTreeView * treeview, char *argv[])
 	print_status(treeview, "xf_ERROR_ICON", strerror(EACCES), " ", 
 			argv[0], NULL);
 	return FALSE;
-    } else g_free(fullpath);
+    } else {
+	    g_free(fullpath);
+	    fullpath=NULL;
+    }
     
-    chdir(get_selected_chdir(treeview));
+    /*printf("DBG:workdir=%s\n",get_selected_chdir(treeview));*/
+    fork_dir=get_selected_chdir(treeview);
+    if (chdir(fork_dir) < 0){
+	    print_diagnostics(treeview, "xf_ERROR_ICON", strerror(EACCES), " ", 
+			fork_dir, NULL);
+	    return FALSE;
+    }
 
 #ifndef G_THREADS_IMPL_NONE
     /* initialize the thread system */
@@ -186,6 +195,10 @@ int runv(GtkTreeView * treeview, char *argv[])
 	printf("\n");
 #endif
 		/*fclose(stdin);*/
+#ifdef DEBUG
+		chdir(fork_dir);
+		printf("DBG: chdir to %s\n",fork_dir);
+#endif
 		execvp(argv[0], argv);
 		_exit(1);
 	    }
@@ -230,7 +243,8 @@ int  parse_runline(char **argv,char *cmd_line,
 	    printf("DBG: argv[%d]=%s\n",i,argv[i]);
 #endif
        if (!strlen(argv[i]))continue;
-       if (is_argv0){
+       if (arg) {
+         if (is_argv0){
 	    is_argv0=FALSE;
  	    if(app != NULL) {
 		if (tok==' ') *app=g_strdup(argv[i]);
@@ -240,7 +254,7 @@ int  parse_runline(char **argv,char *cmd_line,
 	       if (tok==' ') *arg=g_strconcat("-e ",argv[i],NULL);
 	       else *arg=g_strconcat("-e ","\"",argv[i],"\"",NULL);
 	    }
-       } else {
+         } else {
 	       gchar *g;
 	       if (!(*arg)) {
 		  *arg=g_strdup("");
@@ -252,6 +266,7 @@ int  parse_runline(char **argv,char *cmd_line,
 	       }
 	       g_free(*arg);
 	       *arg=g;	
+         }
        }
        if (i == MAX_ARGS-1){
 	     argv[i]=NULL;
@@ -268,7 +283,7 @@ int  parse_runline(char **argv,char *cmd_line,
 			    
 }
 
-int on_run(GtkTreeView * treeview, char *in_cmd, tree_entry_t * en, gboolean in_terminal, gboolean remember, gboolean put_in_history)
+int on_run_path(GtkTreeView * treeview, char *in_cmd, char *path, gboolean in_terminal, gboolean remember, gboolean put_in_history)
 {
     static gchar *gs=NULL;
     gchar *term=NULL;
@@ -302,8 +317,8 @@ int on_run(GtkTreeView * treeview, char *in_cmd, tree_entry_t * en, gboolean in_
 		  else 
 		        cmd_line=g_strconcat(term," -e ",in_cmd,NULL);
 		} else term = NULL;
-		g_free(c);
-		g_free(t);
+		g_free(c);c=NULL;
+		g_free(t);t=NULL;
 	} 
 		
 	if(!term) {
@@ -326,23 +341,23 @@ int on_run(GtkTreeView * treeview, char *in_cmd, tree_entry_t * en, gboolean in_
     i=parse_runline(argv,cmd_line,&app,&arg);
 
     /* command argv done, only need to add the file path */
-    if(en)
+    if(path)
     {
 	    for (j=1;argv[j];j++) {
 		    if (strcmp(argv[j],"\%s")==0){
-			    argv[j]=en->path;
+			    argv[j]=path;
 			    break;
 		    }
 		    if (strstr(argv[j],"\%s")){
-			    gs=g_strconcat(argv[j]," ",en->path,NULL);
-			    g_snprintf(gs,strlen(gs),argv[j],en->path);
+			    gs=g_strconcat(argv[j]," ",path,NULL);
+			    g_snprintf(gs,strlen(gs),argv[j],path);
 			    argv[j]=gs;
 			    break;
 		    }
 
 	    }
 	    if (!argv[j]) {
-		    argv[i++]=en->path;
+		    argv[i++]=path;
 		    argv[i] = NULL;
 	    }
     }
@@ -352,10 +367,11 @@ int on_run(GtkTreeView * treeview, char *in_cmd, tree_entry_t * en, gboolean in_
     if(!runv(treeview, argv))
     {
         g_free(gs);gs=NULL;
-	g_free(app);
-	g_free(arg);
-	g_free(cmd_line);
-	g_free(term);
+	g_free(app);app=NULL;
+	g_free(arg);arg=NULL;
+	g_free(cmd_line);cmd_line=NULL;
+	g_free(term);term=NULL;
+
 	return FALSE;
     }
 
@@ -363,16 +379,15 @@ int on_run(GtkTreeView * treeview, char *in_cmd, tree_entry_t * en, gboolean in_
 	static void save_run_path(char *p);
 	save_run_path(in_cmd);
     }
-    if(en && remember)
+    if(path && remember)
     {
 	char *sfx;
-	if(!en->path) g_assert_not_reached();
-	sfx = strchr(en->path, '.');
+	sfx = strchr(path, '.');
 	if (sfx && strchr(sfx,'/')) sfx=NULL;
 
-	if(!sfx && strlen(en->path) > 1)
+	if(!sfx && strlen(path) > 1)
 	{
-	    sfx = strrchr(en->path, '/');
+	    sfx = strrchr(path, '/');
 	    if(sfx) sfx++;
 	}
 	if(sfx)
@@ -385,16 +400,21 @@ int on_run(GtkTreeView * treeview, char *in_cmd, tree_entry_t * en, gboolean in_
 	}
     }
     g_free(gs);gs=NULL;
-    g_free(app);
-    g_free(arg);
-    g_free(cmd_line);
-    g_free(term);
+    g_free(app);app=NULL;
+    g_free(arg);arg=NULL;
+    g_free(cmd_line);cmd_line=NULL;
+    g_free(term);term=NULL;
     return TRUE;
 }
 
+int on_run(GtkTreeView * treeview, char *in_cmd, tree_entry_t * en, gboolean in_terminal, gboolean remember, gboolean put_in_history)
+{
+	char *path=NULL;
+	if (en && !IS_APP_TYPE(en->type)) path=en->path;
+	return on_run_path(treeview, in_cmd, path, in_terminal, remember, put_in_history);
+}
 
 
-#define CURRENT_RUN_HISTORY "xffm%cxffm.runlist.2.dbh"
 static void save_run_path(char *p)
 {				
     char fname[_POSIX_PATH_MAX];
@@ -404,9 +424,14 @@ static void save_run_path(char *p)
 }
 
 
-static void load_run_list(GList **g)
+GList *set_run_combo(history_combo_info_t *combo_info)
 {
-    char fname[_POSIX_PATH_MAX];
+    tree_details_t *tree_details = get_tree_details(combo_info->treeview);
+    char *p;
+    GtkWidget *w;
+    GtkTreeIter iter;
+    tree_entry_t *en;
+    static char fname[_POSIX_PATH_MAX];
     /* eliminate pre 4.0-beta3 runlist, if any */
     xfce_get_userfile_r(fname, _POSIX_PATH_MAX-1, "xffm%cxffm.runlist",
 		    G_DIR_SEPARATOR);
@@ -426,36 +451,21 @@ static void load_run_list(GList **g)
 	    for (tmp=l;tmp;tmp=tmp->next) {
 		    save_run_path((char *)tmp->data);
 		    g_free((char *)tmp->data);
+		    tmp->data=NULL;
 	    }
 	    g_list_free(l);
     }
-    get_history_list(g,fname,"");
-    return;
-}
+    get_history_list(&run_list,fname,"");
+    combo_info->active_dbh_file=fname;
+    combo_info->list=run_list;
 
-GList *set_run_combo(GtkTreeView * treeview)
-{
-    tree_details_t *tree_details = get_tree_details(treeview);
-    char *p;
-    GtkWidget *w;
-    GtkTreeIter iter;
-    tree_entry_t *en;
-    w = lookup_widget(tree_details->window, "input_combo");
-    load_run_list(&run_list);
-#ifdef DEBUG
-    {
-	    //GList *tmp=run_list;
-	    //for (;tmp;tmp=tmp->next)
-		    //printf("DBG: runlist element=%s\n",(char *)tmp->data);
-    }
-#endif
     if(!run_list) g_assert_not_reached();
-    en = get_selected_entry(treeview, &iter);
+    en = get_selected_entry(combo_info->treeview, &iter);
     if (en) {
 	p = g_strdup(reg_app_by_file(en->path));
     	if (p) run_list = g_list_prepend(run_list, p);
     }
-    gtk_combo_set_popdown_strings(GTK_COMBO(w), run_list);
+    set_limited_combo(combo_info,NULL);
     return run_list;
 }
 
@@ -511,38 +521,35 @@ void double_click_run(tree_details_t *tree_details,
 	}
      }
      g_free(cmd);
+     cmd=NULL;
     }
 
     
      /* filter element is double used for arguments here */
-    if (IS_APP_TYPE(en->type) && en->filter)a=g_strconcat(en->path," ",en->filter,NULL);
-    else  a=g_strdup(en->path);
+    if (IS_APP_TYPE(en->type) && en->filter) a=g_strconcat(en->path," ",en->filter,NULL);
+    else if (IS_EXE(en->type)) a=NULL;
+    else a=g_strdup(en->path);
    
     print_status(tree_details->treeview, "xf_INFO_ICON",
 		   _("Executing"), " : ", 
 		    FILENAME(en), 
 		   NULL);
     on_run(tree_details->treeview, a, en, interm, FALSE,FALSE);
-    g_free(a);
+    g_free(a);a=NULL;
     return;
 }
 
 void double_click_open_with(tree_details_t *tree_details,
 		tree_entry_t *en){
     /* open with */
-    char *name, *sfx = NULL;
+    char *name;
     char arguments[256];
     reg_t *prg;
     name = strrchr(en->path, '/');
-    if(!name)
-	g_assert_not_reached();
-    if(strlen(name) > 1)
-	name++;
-    sfx = strchr(name, '.');
-    if(!sfx)
-	sfx = name;
-    /*printf("DBG:looking for %s\n",sfx); */
-    prg = reg_prog_by_suffix(sfx);
+    if(!name) g_assert_not_reached();
+    if(strlen(name) > 1) name++;
+     /*printf("DBG:looking for %s\n",name); */
+    prg = reg_prog_by_file(name);
     if(prg)
     {
 	sprintf(arguments, "%s %s", prg->app, (prg->arg) ? prg->arg : " ");
