@@ -96,7 +96,10 @@ int open_smb(GtkTreeView * treeview,
 			  checkfiles[i],"\n",
 			  NULL);
 	  return (0);
-	} else g_free(test);
+	} else {
+		g_free(test);
+		test=NULL;
+	}
   }
   gtk_tree_model_get(GTK_TREE_MODEL(treemodel), iter, ENTRY_COLUMN, &en, -1);
   if (IS_ROOT_TYPE(en->type)) {
